@@ -10,6 +10,8 @@ New API for plugins.
 All plugins in Spyder 5+ must inherit from the classes present in this file.
 """
 
+from __future__ import annotations
+
 # Standard library imports
 from collections import OrderedDict
 import inspect
@@ -23,13 +25,13 @@ import warnings
 # Third party imports
 from qtpy.QtCore import QObject, Qt, Signal, Slot
 from qtpy.QtGui import QCursor
-from qtpy.QtWidgets import QApplication
+from qtpy.QtWidgets import QApplication, QMainWindow
 
 # Local imports
 from spyder.api.config.mixins import SpyderConfigurationObserver
 from spyder.api.exceptions import SpyderAPIError
+from spyder.api.fonts import SpyderFontType
 from spyder.api.plugin_registration.mixins import SpyderPluginObserver
-from spyder.api.translations import get_translation
 from spyder.api.widgets.main_widget import PluginMainWidget
 from spyder.api.widgets.mixins import SpyderActionMixin
 from spyder.api.widgets.mixins import SpyderWidgetMixin
@@ -41,11 +43,10 @@ from spyder.utils.image_path_manager import IMAGE_PATH_MANAGER
 
 # Package imports
 from .enum import Plugins
-from .old_api import SpyderPluginWidget
+from ._old_api import SpyderPluginWidget
 
 
-# Localization
-_ = get_translation('spyder')
+# Logging
 logger = logging.getLogger(__name__)
 
 
@@ -149,8 +150,8 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
     IMG_PATH = None
 
     # Control the font size relative to the global fonts defined in Spyder
-    FONT_SIZE_DELTA = 0
-    RICH_FONT_SIZE_DELTA = 0
+    MONOSPACE_FONT_SIZE_DELTA = 0
+    INTERFACE_FONT_SIZE_DELTA = 0
 
     # Define context to store actions, toolbars, toolbuttons and menus.
     CONTEXT_NAME = None
@@ -160,6 +161,13 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
     # disabled. Default: True
     CAN_BE_DISABLED = True
 
+    # Qt Web Widgets may be a heavy dependency for many packagers
+    # (e.g. conda-forge)
+    # We thus ask plugins to declare whether or not they need
+    # web widgets to enhance the distribution of Spyder to users
+    # https://github.com/spyder-ide/spyder/pull/22196#issuecomment-2189377043
+    REQUIRE_WEB_WIDGETS = False
+
     # --- API: Signals -------------------------------------------------------
     # ------------------------------------------------------------------------
     # Signals here are automatically connected by the Spyder main window and
@@ -279,6 +287,17 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
         Unmaximize plugin only if it is not `plugin_instance`.
     """
 
+    sig_mainwindow_state_changed = Signal(object)
+    """
+    This signal is emitted when the main window state has changed (for
+    instance, between maximized and minimized states).
+
+    Parameters
+    ----------
+    window_state: Qt.WindowStates
+        The window state.
+    """
+
     # --- Private attributes -------------------------------------------------
     # ------------------------------------------------------------------------
     # Define configuration name map for plugin to split configuration
@@ -303,7 +322,7 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
         self._actions = {}
         self.is_compatible = None
         self.is_registered = None
-        self.main = parent
+        self.main: QMainWindow = parent
 
         # Attribute used to access the action, toolbar, toolbutton and menu
         # registries
@@ -316,6 +335,9 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
                 parent=parent
             )
 
+            if hasattr(container, '_setup'):
+                container._setup()
+
             if isinstance(container, SpyderWidgetMixin):
                 container.setup()
                 container.update_actions()
@@ -334,9 +356,6 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
 
             self.after_container_creation()
 
-            if hasattr(container, '_setup'):
-                container._setup()
-
         # Load the custom images of the plugin
         if self.IMG_PATH:
             plugin_path = osp.join(self.get_path(), self.IMG_PATH)
@@ -404,7 +423,7 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
         """
         return self._main
 
-    def get_plugin(self, plugin_name, error=True):
+    def get_plugin(self, plugin_name, error=True) -> SpyderPluginV2:
         """
         Get a plugin instance by providing its name.
 
@@ -459,7 +478,7 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
                 dockable_plugins_required.append(plugin_instance)
         return dockable_plugins_required
 
-    def get_conf(self, option, default=NoDefault, section=None):
+    def get_conf(self, option, default=NoDefault, section=None, secure=False):
         """
         Get an option from Spyder configuration system.
 
@@ -472,6 +491,9 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
             Python object.
         section: str
             Section in the configuration system, e.g. `shortcuts`.
+        secure: bool
+            If True, the option will be retrieved securely using the `keyring`
+            Python package.
 
         Returns
         -------
@@ -485,12 +507,18 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
                     'A spyder plugin must define a `CONF_SECTION` class '
                     'attribute!'
                 )
-            return self._conf.get(section, option, default)
+            return self._conf.get(section, option, default, secure=secure)
 
     @Slot(str, object)
     @Slot(str, object, str)
-    def set_conf(self, option, value, section=None,
-                 recursive_notification=True):
+    def set_conf(
+        self,
+        option,
+        value,
+        section=None,
+        recursive_notification=True,
+        secure=False,
+    ):
         """
         Set an option in Spyder configuration system.
 
@@ -510,6 +538,9 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
             changes, then the observers for section `sec` are notified.
             Likewise, if the option `(a, b, c)` changes, then observers for
             `(a, b, c)`, `(a, b)` and a are notified as well.
+        secure: bool
+            If True, the option will be saved securely using the `keyring`
+            Python package.
         """
         if self._conf is not None:
             section = self.CONF_SECTION if section is None else section
@@ -519,11 +550,16 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
                     'attribute!'
                 )
 
-            self._conf.set(section, option, value,
-                           recursive_notification=recursive_notification)
+            self._conf.set(
+                section,
+                option,
+                value,
+                recursive_notification=recursive_notification,
+                secure=secure,
+            )
             self.apply_conf({option}, False)
 
-    def remove_conf(self, option, section=None):
+    def remove_conf(self, option, section=None, secure=False):
         """
         Delete an option in the Spyder configuration system.
 
@@ -533,6 +569,9 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
             Name of the option, either a string or a tuple of strings.
         section: str
             Section in the configuration system.
+        secure: bool
+            If True, the option will be removed securely using the `keyring`
+            Python package.
         """
         if self._conf is not None:
             section = self.CONF_SECTION if section is None else section
@@ -542,7 +581,7 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
                     'attribute!'
                 )
 
-            self._conf.remove_option(section, option)
+            self._conf.remove_option(section, option, secure=secure)
             self.apply_conf({option}, False)
 
     def apply_conf(self, options_set, notify=True):
@@ -677,15 +716,18 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
         return ima.icon(name)
 
     @classmethod
-    def get_font(cls, rich_text=False):
+    def get_font(cls, font_type):
         """
-        Return plain or rich text font used in Spyder.
+        Return one of font types used in Spyder.
 
         Parameters
         ----------
-        rich_text: bool
-            Return rich text font (i.e. the one used in the Help pane)
-            or plain text one (i.e. the one used in the Editor).
+        font_type: str
+            There are three types of font types in Spyder:
+            SpyderFontType.Monospace, used in the Editor, IPython console,
+            and History; SpyderFontType.Interface, used by the entire Spyder
+            app; and SpyderFontType.MonospaceInterface, used by the Variable
+            Explorer, Find, Debugger and others.
 
         Returns
         -------
@@ -694,19 +736,20 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
 
         Notes
         -----
-        All plugins in Spyder use the same, global font. This is a convenience
-        method in case some plugins want to use a delta size based on the
-        default one. That can be controlled by using FONT_SIZE_DELTA or
-        RICH_FONT_SIZE_DELTA (declared in `SpyderPlugin`).
+        All plugins in Spyder use the same, global fonts. In case some a plugin
+        wants to use a delta font size based on the default one, they can set
+        the MONOSPACE_FONT_SIZE_DELTA or INTERFACE_FONT_SIZE_DELTA class
+        constants.
         """
-        if rich_text:
-            option = 'rich_font'
-            font_size_delta = cls.RICH_FONT_SIZE_DELTA
+        if font_type == SpyderFontType.Monospace:
+            font_size_delta = cls.MONOSPACE_FONT_SIZE_DELTA
+        elif font_type in [SpyderFontType.Interface,
+                           SpyderFontType.MonospaceInterface]:
+            font_size_delta = cls.INTERFACE_FONT_SIZE_DELTA
         else:
-            option = 'font'
-            font_size_delta = cls.FONT_SIZE_DELTA
+            raise SpyderAPIError("Unrecognized font type")
 
-        return get_font(option=option, font_size_delta=font_size_delta)
+        return get_font(option=font_type, font_size_delta=font_size_delta)
 
     def get_command_line_options(self):
         """
@@ -736,11 +779,12 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
 
         Notes
         -----
-        This is a method to be able to update localization without a restart.
+        This method needs to be decorated with `staticmethod`.
         """
         raise NotImplementedError('A plugin name must be defined!')
 
-    def get_description(self):
+    @staticmethod
+    def get_description():
         """
         Return the plugin localized description.
 
@@ -751,11 +795,12 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
 
         Notes
         -----
-        This is a method to be able to update localization without a restart.
+        This method needs to be decorated with `staticmethod`.
         """
         raise NotImplementedError('A plugin description must be defined!')
 
-    def get_icon(self):
+    @classmethod
+    def get_icon(cls):
         """
         Return the plugin associated icon.
 
@@ -763,6 +808,10 @@ class SpyderPluginV2(QObject, SpyderActionMixin, SpyderConfigurationObserver,
         -------
         QIcon
             QIcon instance
+
+        Notes
+        -----
+        This method needs to be decorated with `classmethod` or `staticmethod`.
         """
         raise NotImplementedError('A plugin icon must be defined!')
 
@@ -972,8 +1021,8 @@ class SpyderDockablePlugin(SpyderPluginV2):
         self.CONTAINER_CLASS = self.WIDGET_CLASS
         super().__init__(parent, configuration=configuration)
 
-        # Defined on mainwindow.py
-        self._shortcut = None
+        # Shortcut to switch to the plugin. It's defined on the main window
+        self._switch_to_shortcut = None
 
         # Widget setup
         # --------------------------------------------------------------------
@@ -1032,7 +1081,7 @@ class SpyderDockablePlugin(SpyderPluginV2):
         super().after_long_process(message)
         self.get_widget().stop_spinner()
 
-    def get_widget(self):
+    def get_widget(self) -> PluginMainWidget:
         """
         Return the plugin main widget.
         """
@@ -1088,7 +1137,7 @@ class SpyderDockablePlugin(SpyderPluginV2):
         self.get_widget().create_window()
 
     def close_window(self, save_undocked=False):
-        self.get_widget().close_window(save_undocked=save_undocked)
+        self.get_widget()._close_window(save_undocked=save_undocked)
 
     def change_visibility(self, state, force_focus=False):
         self.get_widget().change_visibility(state, force_focus)
