@@ -11,9 +11,10 @@ import copy
 import functools
 
 # Third party imports
-from qtpy.QtGui import QTextCursor, QColor
-from qtpy.QtCore import Qt, QMutex, QMutexLocker
 from diff_match_patch import diff_match_patch
+from qtpy.QtCore import QMutex, QMutexLocker, Qt
+from qtpy.QtGui import QTextCursor, QColor
+from superqt.utils import qdebounced
 
 try:
     from rtree import index
@@ -22,8 +23,8 @@ except Exception:
     rtree_available = False
 
 # Local imports
+from spyder.plugins.editor.api.editorextension import EditorExtension
 from spyder.py3compat import to_text_string
-from spyder.api.editorextension import EditorExtension
 from spyder.utils.snippets.ast import build_snippet_ast, nodes, tokenize
 
 
@@ -89,7 +90,6 @@ class SnippetsExtension(EditorExtension):
         self.starting_position = None
         self.modification_lock = QMutex()
         self.event_lock = QMutex()
-        self.update_lock = QMutex()
         self.node_position = {}
         self.snippets_map = {}
         self.undo_stack = []
@@ -98,7 +98,7 @@ class SnippetsExtension(EditorExtension):
             self.index = index.Index()
 
     def on_state_changed(self, state):
-        """Connect/disconnect sig_key_pressed signal."""
+        """Connect/disconnect editor signals."""
         if state:
             self.editor.sig_key_pressed.connect(self._on_key_pressed)
             self.editor.sig_insert_completion.connect(self.insert_snippet)
@@ -589,7 +589,8 @@ class SnippetsExtension(EditorExtension):
 
     @lock
     def remove_selection(self, selection_start, selection_end):
-        self._remove_selection(selection_start, selection_end)
+        if self.is_snippet_active:
+            self._remove_selection(selection_start, selection_end)
 
     def _remove_selection(self, selection_start, selection_end):
         start_node, _, _ = self._find_node_by_position(*selection_start)
@@ -712,8 +713,10 @@ class SnippetsExtension(EditorExtension):
             text_ids = set([id(token) for token in nearest_text.tokens])
             if node_id not in text_ids:
                 current_node = nearest_text.tokens[-1]
+
         return current_node, nearest_snippet, nearest_text
 
+    @qdebounced(timeout=20)
     def cursor_changed(self, line, col):
         if not rtree_available:
             return
@@ -722,14 +725,16 @@ class SnippetsExtension(EditorExtension):
             self.inserting_snippet = False
             return
 
-        node, nearest_snippet, _ = self._find_node_by_position(line, col)
-        if node is None:
-            ignore = self.editor.is_undoing or self.editor.is_redoing
-            if not ignore:
-                self.reset()
-        else:
-            if nearest_snippet is not None:
-                self.active_snippet = nearest_snippet.number
+        if self.is_snippet_active:
+            node, nearest_snippet, _ = self._find_node_by_position(line, col)
+
+            if node is None:
+                ignore = self.editor.is_undoing or self.editor.is_redoing
+                if not ignore:
+                    self.reset()
+            else:
+                if nearest_snippet is not None:
+                    self.active_snippet = nearest_snippet.number
 
     def reset(self, partial_reset=False):
         self.node_number = 0
@@ -779,7 +784,6 @@ class SnippetsExtension(EditorExtension):
                     self.editor.highlight_selection('code_snippets',
                                                     QTextCursor(cursor),
                                                     outline_color=color)
-                    self.editor.update_extra_selections()
 
     def select_snippet(self, snippet_number):
         cursor = self.editor.textCursor()
